<?php

/*
 * This file is part of the PHP Bench package
 *
 * (c) Daniel Leech <daniel@dantleech.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace PhpBench\Benchmark;

use PhpBench\BenchmarkInterface;

class SubjectBuilder
{
    /**
     * @var int
     */
    public static $subjectIdCounter = 0;

    /**
     * @var Parser
     */
    private $parser;

    /**
     * @param array $subjects Subject whitelist (empty implies all subjects)
     * @param array $parameters Parameter override (empty will use annotated parameters)
     * @param array $groups Group whitelist (empty implies all groups)
     */
    public function __construct()
    {
        $this->parser = new Parser();
    }

    public function buildSubjects(
        BenchmarkInterface $benchmark,
        array $subjectsOverride = null,
        array $groups = null
    ) {
        $reflection = new \ReflectionClass(get_class($benchmark));
        $defaults = $this->parser->parseDoc($reflection->getDocComment());
        $methods = $reflection->getMethods();

        $subjects = array();
        foreach ($methods as $method) {
            if (0 !== strpos($method->getName(), 'bench')) {
                continue;
            }

            // if we have a subject whitelist, only include subjects in that whitelistd
            if ($subjectsOverride && false === in_array($method->getName(), $subjectsOverride)) {
                continue;
            }

            $meta = $this->parser->parseDoc($method->getDocComment(), $defaults);

            if ($groups && 0 === count(array_intersect($groups, $meta['group']))) {
                continue;
            }

            if (empty($meta['revs'])) {
                $meta['revs'] = array(1);
            }

            $subjects[] = new Subject(
                self::$subjectIdCounter++,
                $method->getName(),
                $meta['beforeMethod'],
                $meta['afterMethod'],
                $meta['paramProvider'],
                $meta['iterations'],
                $meta['revs'],
                $meta['group']
            );
        }

        return $subjects;
    }
}
